import os
from abcli.modules import host
from . import NAME
from abcli.logger import logger


screen_width = None
screen_height = None

try:
    if host.is_rpi():
        if not host.is_headless():
            # https://stackoverflow.com/a/14124257
            screen = os.popen("xrandr -q -d :0").readlines()[0]
            screen_width = int(screen.split()[7])
            screen_height = int(screen.split()[9][:-1])
    elif host.is_mac():
        success, output = host.shell(
            "system_profiler SPDisplaysDataType |grep Resolution",
            clean_after=True,
            return_output=True,
        )
        output = [thing for thing in output if thing]
        if success and output:
            screen_width, screen_height = [
                int(thing) for thing in output[-1].split() if thing.isnumeric()
            ]

    elif (
        not host.is_ec2()
        and not host.is_docker()
        and not host.is_github_workflow()
        and not host.is_jupyter()
        and not host.is_aws_batch()
    ):
        from gi.repository import Gdk

        screen = Gdk.Screen.get_default()
        geo = screen.get_monitor_geometry(screen.get_primary_monitor())
        screen_width = geo.width
        screen_height = geo.height
except Exception as e:
    logger.error(f"{NAME}: Failed: {e}.")

used_default = False
if screen_height is None or screen_width is None:
    used_default = True
    screen_height = 480
    screen_width = 640



from functools import reduce
import json
from abcli import string
from abcli.table import Table
from abcli.logger import logger


columns = "keyword,value,timestamp".split(",")


def create():
    """create the cache table.

    Returns:
        bool: success.
    """
    return Table.Create(
        "cache",
        [
            "keyword VARCHAR(1024) NOT NULL",
            "value VARCHAR(4096) NOT NULL",
        ],
    )


def clone(source, destination):
    """clone source in destination.

    Args:
        source (str): source.
        destination (str): destination.

    Returns:
        bool: success.
    """
    lut = search(f"{source}.%")

    lut = {
        "{}.{}".format(
            destination,
            string.after(
                keyword,
                f"{source}.",
            ),
        ): value
        for keyword, value in lut.items()
    }

    return reduce(
        lambda x, y: x and y,
        [write(keyword, value) for keyword, value in lut.items()],
        True,
    )


def read(
    keyword,
    all=False,
    dataframe=False,
    like=False,
    unique=False,
):
    """read keyword form cache.

    Args:
        keyword (str): keyword.
        all (bool, optional): return all values. Defaults to False.
        dataframe (bool, optional): return dataframe. Defaults to False.
        like (bool, optional): like. Defaults to False.
        unique (bool, optional): return unique values. Defaults to False.

    Returns:
        Any: output.
    """
    if dataframe:
        all = True

    table = Table(name="cache")

    if isinstance(keyword, list):
        keyword = ".".join(keyword)

    if not table.connect():
        return None

    success, output = table.execute(
        (
            "SELECT {} FROM ".format(
                ",".join(["c.{}".format(column) for column in columns])
            )
            + "abcli.cache c "
        )
        + (
            (
                "INNER JOIN ( "
                "SELECT keyword, MAX(timestamp) AS max_timestamp "
                "From abcli.cache "
                "GROUP BY keyword "
                ") cm ON c.keyword = cm.keyword AND c.timestamp = cm.max_timestamp "
            )
            if unique
            else ""
        )
        + (
            "WHERE c.keyword {} '{}' ".format("like" if like else "=", keyword)
            + "ORDER BY c.timestamp DESC "
            + "{};".format(
                "" if all else "LIMIT 1",
            )
        )
    )
    if not success:
        return None

    if not table.disconnect():
        return None

    output = [
        {keyword: item for keyword, item in zip(columns, thing)} for thing in output
    ]

    if not dataframe:
        output = [item["value"] for item in output]

    if not all:
        output = "" if not output else output[0]

    if dataframe:
        import pandas as pd

        output = pd.DataFrame.from_dict(output)

    return output


def search(keyword):
    """search cache for keyword.

    Args:
        keyword (str): keyword.

    Returns:
        dict: {keyword:value}
    """
    table = Table(name="cache")

    if isinstance(keyword, list):
        keyword = ".".join(keyword)

    if not table.connect():
        return {}

    success, output = table.execute(
        "SELECT keyword,value FROM abcli.cache "
        f"WHERE keyword like '{keyword}' "
        "ORDER BY timestamp ASC;"
    )

    if success:
        success = table.disconnect()

    return {thing[0]: thing[1] for thing in output} if success else {}


def search_value(value):
    """search for value in cache.

    Args:
        value (str): value.

    Returns:
        List[str]: list of keywords.
    """
    table = Table(name="cache")

    if not table.connect():
        return []

    success, output = table.execute(
        "SELECT keyword,value FROM abcli.cache "
        f"WHERE value = '{value}' "
        "ORDER BY timestamp DESC;"
    )

    if success:
        success = table.disconnect()

    return [thing[0] for thing in output] if success else []


def write(keyword, value):
    """write keyword=value in cache.

    Args:
        keyword (str): keyword.
        value (str): value.

    Returns:
        bool: success.
    """
    table = Table(name="cache")

    if isinstance(keyword, list):
        keyword = ".".join(keyword)

    if isinstance(value, dict):
        value = json.dumps(value)

    if not table.connect():
        return False

    success = table.insert(
        ["keyword", "value"],
        [keyword, value],
    )

    if success:
        success = table.disconnect()

    if success:
        logger.info("cache[{}] <- {}".format(keyword, value))

    return success



import argparse
from . import NAME, Options

list_of_tasks = "choice|get|subset"

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    default="",
    help=list_of_tasks,
)
parser.add_argument(
    "--options",
    type=str,
    default="",
)
parser.add_argument(
    "--subset",
    type=str,
    default="",
)
parser.add_argument(
    "--keyword",
    type=str,
    default="",
)
parser.add_argument(
    "--choices",
    type=str,
    default="",
)
parser.add_argument(
    "--default",
    type=str,
    default="",
)
parser.add_argument(
    "--delim",
    type=str,
    default=",",
)
parser.add_argument(
    "--is_int",
    type=int,
    default=0,
    help="0|1",
)
args = parser.parse_args()

delim = " " if args.delim == "space" else args.delim

success = args.task in list_of_tasks.split("|")
if args.task == "choice":
    options = Options(args.options)

    found = False
    for keyword in args.choices.split(","):
        if options.get(keyword, 0):
            print(keyword)
            found = True
            break

    if not found:
        print(args.default)
elif args.task == "get":
    output = Options(args.options).get(args.keyword, args.default)
    print((int(output) if output else 0) if args.is_int == 1 else output)
elif args.task == "subset":
    options = Options(args.options)
    subset = Options(args.subset)

    print(
        Options(
            {
                keyword: options.get(
                    keyword,
                    value,
                )
                for keyword, value in subset.items()
            }
        ).to_str()
    )
else:
    print(f"-{NAME}: {args.task}: command not found")

if not success:
    print(f"-{NAME}: {args.task}: failed")
